import { Throttler } from "./utils";

type Position = {
    x: number;
    y: number;
};

type Size = { width: number; height: number };

type Coordinate = {
    longitude: number;
    latitude: number;
};

type ZipCodeReverseResponse = {
    nr: number | null;
    navn: string;
};

async function fetchZipCode({
    longitude,
    latitude,
}: Coordinate): Promise<ZipCodeReverseResponse> {
    return fetch(
        `https://api.dataforsyningen.dk/postnumre/reverse?x=${longitude}&y=${latitude}`,
    )
        .then((request) => request.json())
        .then((data) => {
            let zipCode = parseInt(data.nr);
            return {
                ...data,
                nr: isNaN(zipCode) ? null : zipCode,
            } as ZipCodeReverseResponse;
        })
        .catch(() => null as never);
}

function convertPixelsToCoordinate(mouse: Position, map: Size): Coordinate {
    const scalar = { x: 8, y: 3.6 };
    const offset = { x: 6.2, y: 57.93 };
    return {
        longitude: (mouse.x / map.width) * scalar.x + offset.x,
        latitude: Math.abs((mouse.y / map.height) * scalar.y - offset.y),
    };
}

function displayMousePosition(element: HTMLParagraphElement, mouse: Position) {
    element.innerHTML = `Mouse position: <code>(${mouse.x}px, ${mouse.y}px)</code>`;
}

function displayCoords(element: HTMLParagraphElement, coords: Coordinate) {
    element.innerHTML = `Coords: <code>${coords.longitude.toFixed(
        3,
    )}, ${coords.latitude.toFixed(3)}</code>`;
}

function displayZipCode(
    element: HTMLParagraphElement,
    zipCode: number | null,
    name: string,
) {
    element.innerHTML =
        zipCode === null
            ? `Postnummer ikke fundet`
            : `Postnummer: <code>${zipCode}</code>, ${name}`;
}

function setupMap(
    mousePositionElement: HTMLParagraphElement,
    coordsElement: HTMLParagraphElement,
    zipCodeElement: HTMLParagraphElement,
) {
    const mapImg = document.querySelector<HTMLImageElement>("#map")!;
    const fetcher = new Throttler(500);

    mapImg.onmousemove = async (event: MouseEvent) => {
        const mousePosition: Position = { x: event.offsetX, y: event.offsetY };
        displayMousePosition(mousePositionElement, mousePosition);
        const mapSize: Size = {
            width: mapImg.clientWidth,
            height: mapImg.clientHeight,
        };
        const coords = convertPixelsToCoordinate(mousePosition, mapSize);
        displayCoords(coordsElement, coords);
        fetcher.call(async () => {
            const response = await fetchZipCode(coords);
            displayZipCode(zipCodeElement, response.nr, response.navn);
        });
    };

    mapImg.onmouseleave = (_event: MouseEvent) => {
        [mousePositionElement, coordsElement, zipCodeElement].forEach(
            (e) => (e.innerHTML = ""),
        );
    };
}

function setupSearchBar(zipCodeElement: HTMLParagraphElement) {
    const searchBar =
        document.querySelector<HTMLFormElement>("#search-bar");
    const searchInput =
        document.querySelector<HTMLInputElement>("#search-input")!;

    // Prevent typing letters
    searchBar.onkeypress = (event: KeyboardEvent) => {console.log(event);
        event.key !== "Enter" || !isNaN(parseInt(event.key));}

    searchBar.onsubmit = async (event: MouseEvent) => {
        event.preventDefault();

        const inputValue = searchInput.value;
        if (!/^\d+$/.test(inputValue)) return;
        const data = await (
            await fetch(
                `https://api.dataforsyningen.dk/postnumre?nr=${inputValue}`,
            )
        ).json();

        displayZipCode(
            zipCodeElement,
            data.length ? parseInt(data[0]["nr"]) : null,
            data.length ? data[0]["navn"] : null,
        );

    };
}

function main() {
    const [mousePositionElement, coordsElement, zipCodeElement] = [
        "#mouse-position",
        "#coords",
        "#zip-code",
    ].map((id) => document.querySelector<HTMLParagraphElement>(id)!);

    setupSearchBar(zipCodeElement);
    setupMap(mousePositionElement, coordsElement, zipCodeElement);
}

main();
